
/*
  lconf_file.c -- Functions for handling files
  Copyright (C) 2018  Charlie Sale

  This file is part of cconf

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cconf/types.h>
#include <cconf/kvtab.h>
#include <cconf/cconf.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "parser.h"
#include "kvtab-private.h"

static char *
format_output_buffer(char **);

int
cconf_open(cconf_t *lconf, char *path)
{
  int ret;

  if (!lconf)
    return -1;

  // Set path
  if (path)
    lconf->path = strdup(path);
  else
    return -1;

  // open the file
  ret = lcc_kvtab_open(lconf->kvtab, path);
  if (ret < 0)
    return -1;

  // Open the file
  yyrestart(lconf->kvtab->file);

  // Parse file
  yyparse(&lconf->kvtab);

  return 0; // File is opened and the path is copied
}

int
cconf_sync(cconf_t *lconf)
{

#if 0
  int ret;

  lcc_kvtab_deinit(lconf->kvtab);

  lconf->kvtab = lcc_kvtab_new();
  if (!lconf->kvtab)
    return -1;

  // Just run cconf_open again
  return cconf_open(lconf, lconf->path);

#else
# warning "Not yet implemented"
#endif

  return 0;
}

int
cconf_update(cconf_t *lconf)
{

#if 0

  char *buffer = lcc_kvtab_dump(lconf->kvtab);
  if (!buffer)
    return -1;

  // overwrite file
  fclose(lconf->kvtab->file);
  lconf->kvtab->file = fopen(lconf->path, "w"); // overwrite mode
  if (!lconf->kvtab->file)
    return -1;

  size_t size = fwrite(buffer, strlen(buffer), 1, lconf->kvtab->file);
  if (size < strlen(buffer))
    return -1;

  free(buffer);

  fclose(lconf->kvtab->file);
  lconf->kvtab->file = fopen(lconf->path, "r"); // back into read only mode

#else
# warning "Not yet implemented"
#endif

  return 0;
}


static char *
format_output_buffer(char **lines)
{
  // length of array in number of strings
  size_t len = sizeof(lines) / sizeof(*lines);

  char *buffer = malloc(sizeof(lines));
  if (!buffer)
    return NULL;

  for (size_t i = 0; i < len; i++)
      strcat(buffer, lines[i]); // append the string to the buffer
  buffer[len] = '\0';           // add a NULL character

  // NOTE: this needs to be freed
  return buffer;
}
