
/*
  kvtab.c -- Implementation of kvtab
  Copyright (C) 2018  Charlie Sale

  This file is part of cconf

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <config.h>

#include <cconf/types.h>
#include <cconf/kvtab.h>
#include <gl_list.h>
#include <gl_array_list.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include "kvtab-private.h"

lcc_kvtab_t *
lcc_kvtab_new(void)
{
  lcc_kvtab_t *kvtab = malloc(sizeof(lcc_kvtab_t));
  if (!kvtab)
    return NULL;

  kvtab->file = NULL; // file is not yet opened

  kvtab->keys = gl_list_nx_create_empty(GL_ARRAY_LIST, NULL, NULL, NULL, false);
  if (!kvtab->keys)
    {
      free(kvtab);
      return NULL;
    }

  kvtab->vals = gl_list_nx_create_empty(GL_ARRAY_LIST, NULL, NULL, NULL, false);
  if (!kvtab->keys)
    {
      gl_list_free(kvtab->keys);
      free(kvtab);
      return NULL;
    }

  return kvtab;
}

int
lcc_kvtab_open(lcc_kvtab_t *tab, const char *path)
{
  if (!tab->file)
    tab->file = fopen(path, "r"); // open file in read and write mode
  else
    return -2;

  if (!tab->file)
    return -1;

  return 0;
}

int
lcc_kvtab_add(lcc_kvtab_t *tab, char *key, void *val)
{
  if (!tab)
    return -1;

  gl_list_node_t ret;
  printf("[debug] Adding key: %s\n", key);
  ret = gl_list_nx_add_last(tab->keys, key);
  /*  if (!ret)
      return -1; */

  printf("[debug] Adding val: %s\n", val);
  ret = gl_list_nx_add_last(tab->vals, val);
  /*  if (!ret)
      return -1; */

  printf("[debug] sizes: keys[%d] vals[%d]\n", gl_list_size(tab->keys), gl_list_size(tab->vals));

  return 0;
}

void *
lcc_kvtab_get(lcc_kvtab_t *tab, char *key)
{
  if (!tab)
    return NULL;

  printf("[debug] Key passed: %s\n", key);

  size_t loc = gl_list_search(tab->keys, key);
  printf("[debug] Location: %d\n", loc);
  if (loc == -1)
    return NULL;

  void *data = gl_list_get_at(tab->vals, loc);
  if (!data)
    return NULL;

  // FIXME: copy data into new pointer?

  return data;
}

char **
lcc_kvtab_dump(lcc_kvtab_t *tab)
{
#if 1
  printf("[debug] table contents\n");
  gl_list_iterator_t key_iter = gl_list_iterator(tab->keys);
  gl_list_iterator_t val_iter = gl_list_iterator(tab->vals);

  char *key;
  char *val;

  while (gl_list_iterator_next(&key_iter, &key, NULL)
         && gl_list_iterator_next(&val_iter, &val, NULL))
    {
      printf("\t [key] %s : [val] %s\n", key, val);
    }

  return NULL;
#else
# warning "Function not yet implemented"
  return NULL;
#endif
}

void
lcc_kvtab_deinit(lcc_kvtab_t *tab)
{
  fclose(tab->file);
  gl_list_free(tab->keys);
  gl_list_free(tab->vals);
  free(tab);
}
